'use strict';

define([
	'libraries/language',
	'libraries/utils',
	'libraries/permissions',
	'libraries/consts'
], function (LANGUAGE, UTILS, PERMISSIONS, CONSTS) {

	var app = angular.module("JetBackupApp", [
		"ngRoute",
		"ngAnimate",
		"ngSanitize",
		"ngMaterial",
		"ui.bootstrap",
		"permission",
		"permission.ng",
		"angular-loading-bar",
		"oitozero.ngSweetAlert",
		"moment-picker",
		"chart.js",
		"dndLists",
		"angularjs-gauge",
		"googlechart",
		"colorpicker.module"
	]).run(["$http", "PermPermissionStore", function($http, PermPermissionStore) {

		for(var i in PERMISSIONS) {
			if(!/^can/.test(i) && !/^is/.test(i) && !/^has/.test(i)) continue;
			PermPermissionStore.definePermission(i, function(name) {
				if(PERMISSIONS[name] === undefined) return false;
				return PERMISSIONS[name];
			});
		}

	}]).config([
		'$routeProvider', '$httpProvider', '$animateProvider', '$controllerProvider', '$compileProvider', '$filterProvider', '$permissionProvider', '$provide', 'cfpLoadingBarProvider',
		function ($routeProvider, $httpProvider, $animateProvider, $controllerProvider, $compileProvider, $filterProvider, $permissionProvider, $provide, cfpLoadingBarProvider) {
			$animateProvider.classNameFilter(/(action-module)/);
			$permissionProvider.suppressUndefinedPermissionWarning(true);

			cfpLoadingBarProvider.includeSpinner = false;
			cfpLoadingBarProvider.includeBar = true;
			cfpLoadingBarProvider.parentSelector = '#loading-bar-container';

			$httpProvider.interceptors.push('responseObserver');
			$httpProvider.defaults.ignoreLoadingBar = true;

			app.register = {
				controller: $controllerProvider.register,
				directive: $compileProvider.directive,
				filter: $filterProvider.register,
				factory: $provide.factory,
				service: $provide.service
			};

			var basePath = "./app";

			$routeProvider
				.when("/fileManager/:type/:id", { templateUrl: basePath + "/views/fileManager.ptt", data: { permissions: { only: 'canManageFileManager', redirectTo: '/' } } })

				.when("/permissions", { templateUrl: basePath + "/views/permissions.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/alerts", { templateUrl: basePath + "/views/alerts.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/logs", { templateUrl: basePath + "/views/logs.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/queues", { templateUrl: basePath + "/views/queues.ptt", data: { permissions: { only: 'canManageQueue', redirectTo: '/' } } })

				.when("/settings", { templateUrl: basePath + "/views/settings.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/settings/:section", { templateUrl: basePath + "/views/settings.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })

				.when("/accounts", { templateUrl: basePath + "/views/accounts.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/accountManage", { templateUrl: basePath + "/views/accountManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/accountManage/:id", { templateUrl: basePath + "/views/accountManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })

				.when("/backupManager", { templateUrl: basePath + "/views/backupManager.ptt", data: { permissions: { only: 'canManageBackups', redirectTo: '/' } } })
				.when("/backupManager/:section", { templateUrl: basePath + "/views/backupManager.ptt", data: { permissions: { only: 'canManageBackups', redirectTo: '/' } } })

				.when("/backupJobManage/:id", { templateUrl: basePath + "/views/backupJobManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/backupJobManage", { templateUrl: basePath + "/views/backupJobManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/backupJobs", { templateUrl: basePath + "/views/backupJobs.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })

				.when("/destinationManage/:id", { templateUrl: basePath + "/views/destinationManage.ptt", data: { permissions: { only: 'canManageDestinations', redirectTo: '/' } } })
				.when("/destinationManage", { templateUrl: basePath + "/views/destinationManage.ptt", data: { permissions: { only: 'canManageDestinations', redirectTo: '/' } } })
				.when("/destinations", { templateUrl: basePath + "/views/destinations.ptt", data: { permissions: { only: 'canManageDestinations', redirectTo: '/' } } })

				.when("/databaseFilters", { templateUrl: basePath + "/views/databaseFilters.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/databaseFilterManage", { templateUrl: basePath + "/views/databaseFilterManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/databaseFilterManage/:id", { templateUrl: basePath + "/views/databaseFilterManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })

				.when("/databaseManage/:id", { templateUrl: basePath + "/views/databaseManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/databaseManage", { templateUrl: basePath + "/views/databaseManage.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })
				.when("/databases", { templateUrl: basePath + "/views/databases.ptt", data: { permissions: { only: 'canManageBackupJobs', redirectTo: '/' } } })

				.when("/accessTokens/:id", { templateUrl: basePath + "/views/accessTokens.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/accessTokenManage/:id", { templateUrl: basePath + "/views/accessTokenManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })

				.when("/hooks", { templateUrl: basePath + "/views/hooks.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/hookManage", { templateUrl: basePath + "/views/hookManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/hookManage/:id", { templateUrl: basePath + "/views/hookManage.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })

				.when("/plugins", { templateUrl: basePath + "/views/plugins.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })
				.when("/plugin/:id", { templateUrl: basePath + "/views/plugin.ptt", data: { permissions: { only: 'isRoot', redirectTo: '/' } } })

				.when("/404", { templateUrl: basePath + "/views/404.ptt" })

				.when('/license', { templateUrl: basePath + "/views/license.ptt", data: { permissions: { only: 'isLicenseIssue', redirectTo: '/' } } })
				.when('/agreement', { templateUrl: basePath + "/views/agreement.ptt", data: { permissions: { only: 'isAgreement', except: 'isLicenseIssue', redirectTo: '/' } } })
				.when("/showcase", { templateUrl: basePath + "/views/showcase.ptt", data: { permissions: { only: 'isShowcase', except: ['isLicenseIssue', 'isAgreement', 'isDisasterRecovery'], redirectTo: '/' } } })
				.when("/disasterRecovery", { templateUrl: basePath + "/views/disasterRecovery.ptt", data: { permissions: {
					only: 'isDisasterRecovery',
					except: ['isLicenseIssue', 'isAgreement'],
					redirectTo: '/' } } })

				.when('/login', { templateUrl: basePath + "/views/login.ptt", data: { permissions: { except: ['isOTP', 'isLoggedIn'], redirectTo: {isOTP:'/otp',isLoggedIn:'/'} } } })
				.when('/otp', { templateUrl: basePath + "/views/otp.ptt", data: { permissions: { only: 'isOTP', redirectTo: '/' } } })

				.when('/', { templateUrl: basePath + "/views/dashboard.ptt", data: { permissions: {
					only: ['isLoggedIn'],
					except: ['isLicenseIssue', 'isAgreement', 'isDisasterRecovery', 'isShowcase'],
					redirectTo: {
						isLoggedIn: '/login',
						isLicenseIssue: '/license',
						isAgreement: '/agreement',
						isDisasterRecovery: '/disasterRecovery',
						isShowcase: '/showcase'
					}
				}}})
				.otherwise('/404');
		}
	]).controller("baseController",
		["$rootScope", "$scope", "$http", "$location", "$timeout", "SweetAlert", "loader", "meta", "jetapi", "storage", "wizards",
			function($rootScope, $scope, $http, $location, $timeout, SweetAlert, loader, meta, jetapi, storage, wizards) {

				loader.setScope($scope);
				loader.setTimeout(200);
				loader.init();

				$rootScope.LANG = LANGUAGE;
				$rootScope.LANG.t = $scope.LANG.translate;
				$rootScope.LANG.d = $scope.LANG.date;
				$rootScope.UTILS = UTILS;
				$rootScope.PERM = PERMISSIONS;
				$rootScope.CONST = CONSTS;
				$rootScope.account = PAGE.account;
				$rootScope.wizards = wizards;
				$rootScope.primaryURL = CONSTS.BASE_URL + window.location.search + '#!';

				$scope.currYear = new Date().getUTCFullYear();

				$scope.licenseText = PERMISSIONS._licenseText;

				$scope.issueText = '';
				$scope.info = window.PAGE.info;
				$scope.info.version = window.PAGE.system.version;
				$scope.info.tier = window.PAGE.system.tier;
				$scope.total_alerts = window.PAGE.alerts.total;

				$scope.menuItem = 'Dashboard';
				$rootScope.$on('menuItem', function(event, item) { $scope.menuItem = item; });

				$scope.expandedRow = undefined;
				$scope.openActions = function(row) {
					$scope.expandedRow = ($scope.expandedRow !== undefined && $scope.expandedRow === row) ? undefined : row;
				};

				$scope.copyToClipboard = function (text, message) {
					if(message === undefined) message = "Shell command copied to your clipboard";
					var copyElement = document.createElement("textarea");
					copyElement.style.position = 'fixed';
					copyElement.style.opacity = '0';
					copyElement.textContent = text;
					var body = document.getElementsByTagName('body')[0];
					body.appendChild(copyElement);
					copyElement.select();
					document.execCommand('copy');
					body.removeChild(copyElement);
					SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(message));
				};

				$scope.collapseExpandMenuStatus = localStorage.getItem('menuState');
				if($scope.collapseExpandMenuStatus === null) $scope.collapseExpandMenuStatus = 'collapse';

				$scope.collapseExpandMenu = function() {
					$scope.collapseExpandMenuStatus = $scope.collapseExpandMenuStatus === 'collapse' ? 'expand' : 'collapse';
					localStorage.setItem('menuState', $scope.collapseExpandMenuStatus);
				};

				$scope.changeView = function(view){
					$location.path(view);
				};

				$scope.changeViewSorted = function(view, storage_id, field, dir) {
					meta.new(storage_id);
					meta.setSortBy(field);
					meta.setSortDirection(dir);
					$scope.changeView(view);
				};

				$scope.logout = function () {
					jetapi.logout({}, function (response) {
						if(response.success) window.location = '/';
						else SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
					});
				};
			}
		]
	).factory('responseObserver', ["$q", function responseObserver($q) {
		return {
			responseError: function(errorResponse) {
				switch (errorResponse.status) {
					case -1:
					case 403:
					case 500:
						location.reload();
					break;
				}
				return $q.reject(errorResponse);
			}
		};
	}]);

	return app;
});