'use strict';

define([
	'app',
	'libraries/language',
	'libraries/utils'
], function(app, LANGUAGE, UTILS) {
	app.controller("accessTokens",
		["$rootScope", "$scope", "$routeParams", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $routeParams, $timeout, jetapi, meta, SweetAlert) {

				$rootScope.$emit('menuItem', 'Users');

				$scope.tokens = [];
				$scope.loadingTokens = false;
				$scope.tokens_user = $routeParams.id;

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("tokens");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["ip","created"]);
				meta.setTotalItems($scope.tokens.length);

				$scope.onClickDelete = function(token) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This access token will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteAccessToken({ _id: token._id }, function(response) {

								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									return;
								}

								SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Access Token delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingTokens = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						filter: meta.getFilter(),
						find: { user: $routeParams.id }
					};

					$scope.tokens = [];

					jetapi.listAccessTokens(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.access_tokens);
						$scope.tokens = response.data.access_tokens;
						$scope.loadingTokens = false;
					});
				};

				$timeout($scope.fetchData);

			}
		]
	);
});