'use strict';

define([
	'app',
	'libraries/language',
	'libraries/consts'
	], function(app, LANGUAGE, CONSTS) {
	app.controller("accountManage",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$uibModal", "jetapi", "meta", "utils", "SweetAlert", "cfpLoadingBar",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $uibModal, jetapi, meta, utils, SweetAlert, cfpLoadingBar) {
				$rootScope.$emit('menuItem', 'Accounts');

				$scope.details = {};
				$scope.saveData = {
					_id: '',
					email: '',
					ips: []
				};

				$scope.saving = false;

				$scope.status = undefined;
				$scope.changed = false;
				$scope.cancelled = false;
				$scope.ip = '';

				$scope.addIP = function () {
					if(!$scope.ip) {
						SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("No IP was provided.", $scope.ip));
						return;
					}
					if($scope.validateIP($scope.ip)) $scope.saveData.ips.push($scope.ip);
					else SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The provided IP (\"%s\") is invalid.", $scope.ip));
					$scope.ip = '';
				};

				$scope.validateIP = function (ip) {
					return CONSTS.IP_FILTER.test(ip);
				};

				$scope.toggleOTP = function() {
					$scope.saveData.otp = $scope.saveData.otp ? 0 : 1;
				};

				$scope.addMultiIPs = function() {

					$scope.listTitle = LANGUAGE.translate("Whitelisted IPs");
					$scope.listData = $scope.saveData.ips.join("\n");

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: './app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while($scope.saveData.ips.length) $scope.saveData.ips.pop();
						records = records.split("\n");
						var invalidIPs = [];
						for(var i = 0; i < records.length; i++) {
							if($scope.validateIP(records[i])) $scope.saveData.ips.push(records[i]);
							else invalidIPs.push(records[i]);
						}

						if(invalidIPs.length) SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The following IPs (\"%s\") is invalid.", invalidIPs.join(", ")))
					}, function() {});
				};

				$scope.fetchAccountData = function(account_id) {

					jetapi.getAccount({ _id: account_id }, function(response) {
						$scope.details = response.data;
						$scope.saveData = utils.duplicateObject($scope.details);
					});
				};

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details, ['_id']);
				}, true);

				$scope.$on('$destroy', function() {
					if($scope.changed && !$scope.cancelled)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/accounts');
				};

				$scope.saveChanges = function(apply) {

					if($scope.saving) return;

					$scope.status = undefined;
					$scope.saving = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					jetapi.manageAccount(apiParams, function(response) {

						$scope.saving = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.saveData._id = response.data._id;
							$scope.details = utils.duplicateObject($scope.saveData);
							if(apply) $location.path('/accountManage/' + response.data._id);
							else $location.path('/accounts');
							SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: response.success ? "success" : "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};

				var account_id = $routeParams.id;
				if(account_id) $timeout($scope.fetchAccountData(account_id));
				else cfpLoadingBar.complete();
			}
		]
	);
});