'use strict';

define([
	'app',
	'libraries/consts',
	'libraries/language'
	], function(app, CONSTS, LANGUAGE) {
	app.controller("accounts",
		["$rootScope", "$scope", "$location", "$timeout", "$uibModal", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, $uibModal, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Accounts');

				$scope.accounts = [];
				$scope.loadingAccounts = false;

				$scope.loaders = {
					delete: false
				};

				var metaAcct = meta.new("accounts");
				$scope.meta = metaAcct;
				$scope.metaData = metaAcct.getData();

				metaAcct.setSortReverse(false);
				if(!metaAcct.getSortBy()) metaAcct.setSortBy("username");
				if(!metaAcct.getSortDirection()) metaAcct.setSortDirection("asc");
				metaAcct.setSortFields(["domain", "ip", "username", "disk_space_usage", "disk_files_usage", "backup_stats.127", "partition", "owner"]);
				metaAcct.setTotalItems($scope.accounts.length);

				$scope.onClickDelete = function(account) {

					if($scope.saveing) return;
					$scope.saveing = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This user will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteAccount({ _id: account._id }, function(response) {

								$scope.loaders.delete = false;
								$scope.saveing = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									return;
								}

								SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("User delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingAccounts = true;

					var apiParams = {
						sort: {},
						skip: metaAcct.getSkip(),
						limit: metaAcct.getPageSize(),
						find: { type: $scope.filter },
						filter: metaAcct.getFilter()
					};

					apiParams.sort[metaAcct.getSortBy()] = metaAcct.getSortDirectionInt();

					$scope.accounts = [];

					jetapi.listAccounts(apiParams, function(response) {
						metaAcct.setTotalItems(response.data.total);
						metaAcct.calculate(response.data.accounts);
						$scope.accounts = response.data.accounts;
						$scope.loadingAccounts = false;
					});
				};

				$timeout($scope.fetchData);
	}]);
});