'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions',
	'libraries/consts'
	], function(app, LANGUAGE, PERMISSIONS, CONSTS) {
	app.controller("backupJobManage",
		["$rootScope", "$scope", "$routeParams", "$templateCache", "$route", "$location", "$timeout", "$uibModal", "$q", "jetapi", "meta", "utils", "SweetAlert",
			function ($rootScope, $scope, $templateCache, $route, $routeParams, $location, $timeout, $uibModal, $q, jetapi, meta, utils, SweetAlert) {
				$rootScope.$emit('menuItem', 'BackupJobs');

				$scope.cancelled = false;

				$scope.saveData.backuptypes = CONSTS.BACKUP_TYPE_DIRECTORIES;

				$scope.flags = [
					{ label: LANGUAGE.translate("Incremental"), value: CONSTS.BACKUP_FLAG_INCREMENTAL },
					{ label: LANGUAGE.translate("Archived"), value: CONSTS.BACKUP_FLAG_ARCHIVED },
					{ label: LANGUAGE.translate("Compressed"), value: CONSTS.BACKUP_FLAG_COMPRESSED }
				];

				$scope.types = [
					{ label: LANGUAGE.translate("Directories"), value: CONSTS.BACKUP_TYPE_DIRECTORIES },
					{ label: LANGUAGE.translate("Databases"), value: CONSTS.BACKUP_TYPE_DATABASES },
					{ label: LANGUAGE.translate("Replicate"), value: CONSTS.BACKUP_TYPE_REPLICATE }//,
					//{ label: LANGUAGE.translate("Disaster Recovery (BMR)"), value: CONSTS.BACKUP_TYPE_BMR }
				];

				if($scope.jobWizard && !$scope.wizardData.jobId){
					$scope.saveData.name = 'JetBackup_' + parseInt(new Date().getTime()%10000);
				}

				var getDestinationConditions = function(destination, type) {
					type = parseInt(type);

					if(['Local','SSH','RsyncNet','FTP','SFTP','AmazonS3'].indexOf(destination) >= 0)
						return [CONSTS.BACKUP_TYPE_REPLICATE, CONSTS.BACKUP_TYPE_DIRECTORIES, CONSTS.BACKUP_TYPE_DATABASES].indexOf(type) >= 0;

					if(['Backblaze','Dropbox','GoogleDrive'].indexOf(destination) >= 0)
						return CONSTS.BACKUP_TYPE_DIRECTORIES == type;//[CONSTS.BACKUP_TYPE_DIRECTORIES, CONSTS.BACKUP_TYPE_DATABASES].indexOf(type) >= 0;

					if(destination === 'BMR')
						return type === CONSTS.BACKUP_TYPE_BMR;
				};

				var recheckType = function(destination) {
					var types = [CONSTS.BACKUP_TYPE_DIRECTORIES, CONSTS.BACKUP_TYPE_DATABASES, CONSTS.BACKUP_TYPE_REPLICATE, CONSTS.BACKUP_TYPE_BMR];
					for(var i =0; i < types.length; i++) {
						if(getDestinationConditions(destination, types[i])) {
							$scope.saveData.backuptypes = types[i];
							break;
						}
					}
				};

				$scope.typeDisabled = function(type) {
					var destination = $scope.destinationsIds[$scope.saveData.destination];
					if(destination === undefined) return false;
					var result = getDestinationConditions(destination.type, type);
					if(type == $scope.saveData.backuptypes && !result) recheckType(destination.type);
					return !result;
				};

				var getStructureConditions = function(destination, type, flag) {
					type = parseInt(type);
					flag = parseInt(flag);

					if(type === CONSTS.BACKUP_TYPE_DATABASES)
						return (flag === CONSTS.BACKUP_FLAG_COMPRESSED);

					if(type === CONSTS.BACKUP_TYPE_BMR || type === CONSTS.BACKUP_TYPE_REPLICATE)
						return (flag === CONSTS.BACKUP_FLAG_INCREMENTAL);

					if(['Local','SSH','RsyncNet','FTP','SFTP','AmazonS3'].indexOf(destination) >= 0)
						return true;

					if(['Backblaze','Dropbox','GoogleDrive'].indexOf(destination) >= 0)
						return [CONSTS.BACKUP_FLAG_ARCHIVED, CONSTS.BACKUP_FLAG_COMPRESSED].indexOf(flag) >= 0;

					if(destination === 'BMR')
						return flag === CONSTS.BACKUP_FLAG_INCREMENTAL;

					return false;
				};

				var recheckFlag = function(destination, type) {
					var flags = [CONSTS.BACKUP_FLAG_INCREMENTAL, CONSTS.BACKUP_FLAG_ARCHIVED, CONSTS.BACKUP_FLAG_COMPRESSED];
					for(var i =0; i < flags.length; i++) {
						if(getStructureConditions(destination, type, flags[i])) {
							$scope.saveData.flag = flags[i];
							break;
						}
					}
				};

				$scope.flagDisabled = function(flag) {
					var destination = $scope.destinationsIds[$scope.saveData.destination];
					if(destination === undefined) destination = { type: "SSH" };
					var result = getStructureConditions(destination.type, $scope.saveData.backuptypes, flag);
					if(flag == $scope.saveData.flag && !result) recheckFlag(destination.type, $scope.saveData.backuptypes);
					return !result;
				};

				$scope.addListRow = function (value, list) {
					if($scope[value] === undefined || !$scope[value].trim()) return;
					if($scope.validateListPath($scope[value].trim(), value === 'excluderow')) list.push($scope[value].trim());
					else SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The provided path (\"%s\") is invalid. The path must start with a \"/\" and can't have trailing \"/\"" + (value !== 'excluderow' ? ", also patterns is not allowed." : '.'), $scope[value].trim()));
					$scope[value] = '';
				};

				$scope.fileBrowse = function() {

					var uibModel = $uibModal.open({
						animation: true,
						size: "lg",
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: './app/views/fileBrowse.ptt',
						controller: 'fileBrowse',
						resolve: {
							listPaths: function () { return $scope.saveData.includelist; }
						}
					});

					uibModel.result.then(function(paths) {
						$scope.saveData.includelist = paths;
					}, function() {});
				};

				$scope.addMultiListRow = function(type, list) {

					if(type == 'include') {
						$scope.listTitle = LANGUAGE.translate("Directories and Files to include");
						$scope.listData = $scope.saveData.includelist.join("\n");
					} else {
						$scope.listTitle = LANGUAGE.translate("Directories and Files to excludes");
						$scope.listData = $scope.saveData.excludelist.join("\n");
					}

					$scope.listUIB = $uibModal.open({
						animation: true,
						scope: $scope,
						ariaLabelledBy: 'modal-title',
						ariaDescribedBy: 'modal-body',
						templateUrl: './app/views/listSelection.ptt'
					});

					$scope.listUIB.result.then(function(records) {
						while(list.length) list.pop();
						records = records.split("\n");
						var invalidPaths = [];
						for(var i = 0; i < records.length; i++) {
							if($scope.validateListPath(records[i], type === 'exclude')) list.push(records[i]);
							else invalidPaths.push(records[i]);
						}

						if(invalidPaths.length) SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("The following paths (\"%s\") is invalid. The path must start with a \"/\" and can't have trailing \"/\"" + (type === 'include' ? ", also patterns is not allowed." : '.'), invalidPaths.join(", ")))
					}, function() {});
				};

				$scope.$watch('saveData.destination', function () {
					console.log($scope.saveData.destination);
				});
				$scope.$on('validateDestination', function(event, data) {
					data.callback(data.destination);

					/*

					console.log($scope.saveData);
					console.log($scope.saveData.backuptypes);
					if(
						($scope.saveData.backuptypes == CONSTS.BACKUP_TYPE_BMR && data.destination.dr_destination) ||
						($scope.saveData.backuptypes != CONSTS.BACKUP_TYPE_BMR && !data.destination.dr_destination)
					) {
						data.callback(data.destination);
					}
					*/
				});

				$scope.$on('fetchJobData', function(event, data) {
					$scope.fetchJobData(data._id, data.callback);
				});

				$scope.fetchJobData = function(job_id, callback) {

					jetapi.getBackupJob({ '_id': job_id }, function(response) {
						if(!response.success) {
							$scope.changed = false;
							SweetAlert.error(LANGUAGE.translate('Error!'), LANGUAGE.translate("Invalid backup job id"));
							$location.path('/backupJobs');
							return;
						}
						$scope.details = response.data;

						if($scope.details.backuptypes < CONSTS.BACKUP_TYPE_DIRECTORIES) $scope.details.backuptypes = CONSTS.BACKUP_TYPE_DIRECTORIES;

						callback($scope.details);
					});

				};

				$scope.$on('$destroy', function() {

					if($scope.changed && !$scope.cancelled && !$scope.jobWizard)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.cancel = function() {
					$scope.cancelled = true;
					$location.path('/backupJobs');
				};

				$scope.saveChanges = function(apply, callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = utils.saveParams($scope.saveData, $scope.details);
					if(apiParams.time_parsed) apiParams.time = $scope.parseTime(apiParams.time_parsed);
					apiParams.action = $scope.details._id ? 'modify' : 'create';

					$scope.validateSchedules($scope.saveData.schedules, function(isConfirm) {
						if(isConfirm) {
							jetapi.manageBackupJob(apiParams, function(response) {

								$scope.saveing = false;
								if(callback !== undefined && typeof callback === 'function') callback(response);

								if(response.success)
								{
									$scope.details = utils.duplicateObject($scope.saveData);
									$scope.updateData($scope.details);

									if(callback === undefined) {
										if (apply) $location.path('/backupJobManage/' + response.data._id);
										else $location.path('/backupJobs');
										SweetAlert.swal(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message), "success");
									}
								}
								else
								{
									$scope.status = {
										message: LANGUAGE.translate(response.message),
										type: response.success ? "success" : "danger",
										closeable: false,
										ttl: 10000
									};
								}
							});
						} else $scope.saveing = false;
					});
				};

				//Handle backupJobWizard broadcast
				$scope.$on($scope.wizardName, function (env, data) {
					$scope.saveData.destination = data.destinationId;
					$scope.saveData.disabled = 1;
					$scope.saveChanges(false, function (response) {
						$scope.process(response);
						// $scope.$emit("onWizardResponse", response);
					});
				});


			}
		]
	);

});