'use strict';

define([
	'app',
	'libraries/language'
	], function(app, LANGUAGE) {
	app.controller("databases",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Databases');

				$scope.databases = [];
				$scope.jobs = {};
				$scope.loadingDatabases = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = '';
				$scope.filterOptions = [
					{ label: LANGUAGE.translate('All Databases'), value: '' },
					{ label: LANGUAGE.translate('MySQL'), value: 'MySQL' },
					{ label: LANGUAGE.translate('PostgreSQL'), value: 'PostgreSQL' },
					{ label: LANGUAGE.translate('MongoDB'), value: 'MongoDB' }
				];

				$scope.types = {
					'MySQL': 'MySQL',
					'PostgreSQL': 'PostgreSQL',
					'MongoDB': 'MongoDB',
				};

				$scope.loaders = {
					state: false,
					validation: false,
					dr: false,
					hide: false,
					delete: false
				};

				meta = meta.new("databases");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("asc");
				meta.setSortFields(["name","type","jobs_count"]);
				meta.setTotalItems($scope.databases.length);

				$scope.onClickDelete = function(database) {

					if($scope.saveing) return;

					if(database.jobs_count > 0)
					{
						SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate("This database has an assigned jobs, Please remove those jobs before deleting this database."));
						return false;
					}

					$scope.saveing = true;
					$scope.loaders.delete = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This database will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteDatabaseServer({ _id: database._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
								}
								else
								{
									SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
								}

							});
						}
						else
						{
							$scope.saveing = false;
							$scope.loaders.delete = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Database delete cancelled"));
						}

					});
				};

				$scope.validateDatabase = function(database) {

					if($scope.saveing) return;
					$scope.saveing = true;
					$scope.loaders.validation = true;

					jetapi.validateDatabaseServer({ _id: database._id }, function(response) {

						$scope.saveing = false;
						$scope.loaders.validation = false;

						if(response.success) SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
						else SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));

					});

				};

				$scope.listJobs = function(callback) {

					jetapi.listBackupJobs({}, function(response) {

						if(!response.success) {
							if(callback !== undefined) callback();
							return;
						}

						for(var i = 0; i < response.data.jobs.length; i++) {
							if(response.data.jobs[i].database === undefined || $scope.jobs[response.data.jobs[i].database] === undefined) continue;
							$scope.jobs[response.data.jobs[i].database].push(response.data.jobs[i]);
						}

						console.log($scope.jobs);
						if(callback !== undefined) callback();
					});
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingDatabases = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.databases = [];

					jetapi.listDatabaseServers(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.databases);
						$scope.databases = response.data.databases;
						for(var i = 0; i < $scope.databases.length; i++) $scope.jobs[$scope.databases[i]._id] = [];
						$scope.listJobs(function () { $scope.loadingDatabases = false; });
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});