'use strict';

define([
		'app',
		'libraries/consts',
		'libraries/language'
	], function(app, CONSTS, LANGUAGE) {

		app.controller("fileManager",
			["$rootScope", "$scope", "$interval", "$timeout", "jetapi", "meta", "SweetAlert", "$window", "$routeParams", "$location",
				function($rootScope, $scope, $interval, $timeout, jetapi, meta, SweetAlert, $window, $routeParams, $location) {
					$rootScope.$emit('menuItem', 'BackupManager');

					$scope.type = $routeParams.type;
					$scope.typeId = $routeParams.id;
					$scope.typeDetails = { name: '' };
					$scope.files = [];
					$scope.loadingFiles = false;
					$scope.currentPath = '';
					$scope.breadcrumbs = [];
					$scope.filesIndex = {};
					$scope.actionStatus = undefined;
					$scope.actionModule = undefined;
					$scope.backup_type = 2;
					$scope.restoringBackup = false;
					$scope.downloadingBackup = false;
					$scope.downloads = [];
					$scope.restores = [];
					$scope.downloadsTmp = [];
					$scope.queues = [];
					$scope.isDirectories = false;
					var statusInterval;

					$scope.clearStatus = function() {
						$scope.actionStatus = undefined;
					};

					$scope.cancelAction = function() {
						$scope.actionStatus = undefined;
						$scope.actionModule = undefined;
					};

					$scope.changeView = function(view){
						$location.path(view); // path not hash
					};

					$scope.onClickRestore = function() {
						var selected = getSelected();

						if(!selected.length)
						{
							$scope.actionStatus = {
								message: "Please select files to restore",
								type: "danger",
								closeable: true,
								ttl: 10000
							};
							return;
						}

						$scope.actionModule = ($scope.actionModule === 'restore') ? undefined : 'restore';
					};

					$scope.onClickRestoreConfirm = function() {

						$scope.restoringBackup = true;
						$scope.actionStatus = undefined;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['files'] = getSelected();

						jetapi.addQueueRestore(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								$scope.restoringBackup = false;
								return false;
							}

							$scope.queues.push(response.data._id);

							$scope.restoringBackup = false;
							$scope.actionModule = undefined;
							$scope.uncheckAll();
							$scope.getRestores();

							SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
							return true;
						});
					};

					$scope.onClickDownload = function() {

						var selected = getSelected();

						if(!selected.length)
						{
							$scope.actionStatus = {
								message: "Please select files to download",
								type: "danger",
								closeable: true,
								ttl: 10000
							};

							return;
						}

						$scope.actionModule = ($scope.actionModule === 'download') ? undefined : 'download';
					};

					$scope.onClickDownloadConfirm = function() {

						$scope.downloadingBackup = true;
						$scope.actionStatus = undefined;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;
						apiParams['files'] = getSelected();

						jetapi.addQueueDownload(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								$scope.actionStatus = { type: "danger", message: LANGUAGE.translate(response.message) };
								$scope.downloadingBackup = false;
								return false;
							}

							$scope.queues.push(response.data._id);

							$scope.downloadingBackup = false;
							$scope.actionModule = undefined;
							$scope.uncheckAll();
							$scope.getDownloads();

							SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
							return true;
						});
					};

					var getSelected = function() {

						var selected = [];

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked)
								{
									selected.push($scope.filesIndex[i].data.files[j].path);
								}
							}
						}

						return selected;
					};

					$scope.canPerformAction = function () {

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked)
								{
									return false;
								}
							}
						}

						return true;
					};

					$scope.isDisabled = function() {
						var file = this.file.parent;
						while(!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					$scope.isChecked= function() {
						var file = this.file;
						while(!file.checked && file.parent) file = file.parent;
						return file.checked;
					};

					$scope.isChildrenChecked= function() {
						var file = this.file;
						return !file.checked && file.children;
					};

					$scope.calculateInput = function(value) {

						if(value === undefined)
						{
							var file = this.file;
							while(!file.checked && file.parent) file = file.parent;
							return file.checked;
						}

						this.file.checked = value;

						var file = this.file;

						while(file.parent !== undefined) {
							file = file.parent;

							if(file.children === undefined) file.children = 0;

							if(value) file.children++;
							else file.children--;
						}
					};

					var manageBreadcrumbs = function(file) {
						if(file.path === '/') file.name = '/' + $scope.type + '-root';
						$scope.breadcrumbs.unshift(file);
						if(file.parent !== undefined) manageBreadcrumbs(file.parent);
					};

					var handleResponse = function(response, file) {
						$scope.filesIndex[file.path] = response;
						$scope.files = response.data.files;
						$scope.loadingFiles = false;
					};

					$scope.fetch = function(file) {

						if(file.path === undefined) file.path = '/';

						$scope.loadingFiles = true;
						$scope.files = [];

						$scope.breadcrumbs = [];
						manageBreadcrumbs(file);

						if($scope.filesIndex[file.path] !== undefined)
						{
							handleResponse($scope.filesIndex[file.path], file);
							return;
						}

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;
						apiParams['path'] = file.path;

						jetapi.fileManager(apiParams, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							}

							for(var i = 0; i < response.data.files.length; i++)
							{
								response.data.files[i].path = file.path + (file.path === '/' ? '' : '/') + response.data.files[i].name;
								response.data.files[i].parent = file;
							}

							handleResponse(response, file);
							return true;
						});

					};

					$scope.uncheckAll = function () {

						for(var i in $scope.filesIndex)
						{
							for(var j in $scope.filesIndex[i].data.files)
							{
								if($scope.filesIndex[i].data.files[j] != null && $scope.filesIndex[i].data.files[j].checked !== undefined && $scope.filesIndex[i].data.files[j].checked)
								{
									$scope.filesIndex[i].data.files[j].checked = false;
								}
							}
						}

					};

					$scope.directDownload = function(download) {
						window.location = '/download/?_id=' + download.download_id;
					};

					$scope.getDownloads = function () {
						$scope.downloads = [];
						$scope.loadingDownloads = true;

						var apiParams = {};
						apiParams['_id'] = $scope.typeId;
						apiParams['type'] = $scope.type;

						jetapi.getBackupDownloads(apiParams, function(response) {

							if (!response.success) {
								$scope.downloads = [];
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								$scope.loadingDownloads = false;
								return false;
							}

							for(var i = 0; i < response.data.downloads.length; i++) {
								$scope.downloads.push(response.data.downloads[i]);
								if ($scope.downloads[i].ready) $scope.downloads[i].status = 100;
								else $scope.downloads[i].status = 1;
							}
							var find = {status:{'$gt':100}};
							jetapi.listQueueItems({ type: CONSTS.QUEUE_TYPE_DOWNLOAD, snap_item_id: $scope.typeId, find: find }, function(response) {

								if(!response.success) {
									SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
									$scope.loadingRestores = false;
									return false;
								}

								for(var i = 0; i < response.data.queue.length; i++)
								{
									var data = response.data.queue[i];

									var filename = '';
									if(data.status > 100) filename = LANGUAGE.translate("Download Failed");
									else filename = LANGUAGE.translate("Download in Progress");

									$scope.downloads.push({
										queue_id: data._id,
										status: data.status,
										ready: !(parseInt(data.status) > 100 || parseInt(data.status) < 100),
										filename: filename,
										download_id: data.download_id
									});
								}

								$scope.loadingDownloads = false;
								return true;
							});

							return true;
						});
					};

					$scope.getRestores = function () {

						$scope.loadingRestores = true;

						jetapi.listQueueItems({ type: CONSTS.QUEUE_TYPE_RESTORE, snap_item_id: $scope.typeId }, function(response) {

							$scope.restores = [];

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								$scope.loadingRestores = false;
								return false;
							}

							for(var i = 0; i < response.data.queue.length; i++)
							{
								var data = response.data.queue[i];

								var text = '';
								if(data.status === 100) text = "Restore Completed";
								else if(data.status > 100) text = "Restore Failed";
								else text = "Restore in Progress";

								$scope.restores.push({
									_id: data._id,
									ready: data.status >= 100,
									status: data.status,
									created: data.created,
									text: LANGUAGE.translate(text)
								});
							}

							$scope.loadingRestores = false;
							return true;
						});
					};

					$scope.inQueues = function(_id) {
						for(var i = 0; i < $scope.queues.length; i++) {
							if($scope.queues[i] == _id) return i;
						}

						return null;
					};

					$scope.checkStatus = function () {

						statusInterval = $interval(function() {

							jetapi.listQueueItems({ snap_item_id: $scope.typeId }, function(response) {

								if (!response.success) {
									SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
									return false;
								}

								// $scope.getDownloads();////////////////////REMOVE THIS///////////////////

								var reload = false;
								for(var i = 0; i < response.data.queue.length; i++) {
									var data = response.data.queue[i];
									var index = $scope.inQueues(data._id);

									if(index !== null) {
										if(data.status >= 100) {
											$scope.queues.splice(index, 1);
											reload = true;
										}
									} else if(data.status < 100) {
										$scope.queues.push(data._id);
									}
								}

								if(reload) {
									$scope.getDownloads();
									$scope.getRestores();
								}

								return true;
							});

						}, 3000);
					};

					$scope.getDestination = function() {

						jetapi.getDestination({ _id: $scope.typeId }, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							}

							$scope.typeDetails = response.data;
							return true;
						});
					};

					$scope.getBackup = function() {

						jetapi.getBackup({ _id: $scope.typeId }, function(response) {

							if(!response.success) {
								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							}

							$scope.typeDetails = response.data;
							$scope.isDirectories = (CONSTS.BACKUP_TYPE_DIRECTORIES & $scope.typeDetails.type) != 0;
							return true;

						});
					};

					$scope.$on('$destroy', function() {
						if(angular.isDefined(statusInterval)) $interval.cancel(statusInterval);
					});

					$scope.init = function () {

						$scope.checkStatus();

						switch($scope.type)
						{
							case 'backup':
								$scope.getBackup();
								$scope.getRestores();
							break;
							case 'destination':
								$scope.getDestination();
							break;
						}

						$scope.getDownloads();
						$scope.fetch({path: '/'});
					};

					$timeout($scope.init());
				}]
		);

	}
);