'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function(app, LANGUAGE, PERMISSIONS) {
	app.controller("logs",
		["$rootScope", "$scope", "$uibModal", "$location", "$timeout", "jetapi", "meta", "filter", "SweetAlert",
			function ($rootScope, $scope, $uibModal, $location, $timeout, jetapi, meta, filter, SweetAlert) {
				$rootScope.$emit('menuItem', 'Logs');

				$scope.logs = [];
				$scope.loadingLogs = false;
				$scope.showPager = true;
				$scope.checkall = false;
				$scope.checked = false;

				var logsFilter = filter.new("logs_filter");
				if(!logsFilter.getFilter()) logsFilter.setFilter(0);
				var filterSession = logsFilter.getFilter();
				$scope.filter = filterSession;
				// removed selected after first display
				logsFilter.setFilter(0);

				var logsSubFilter = filter.new("logs_subfilter");
				$scope.subfilter = logsSubFilter.getFilter();
				// removed selected after first display
				logsSubFilter.setFilter("");

				$scope.subfilterOptions = [];
				$scope.filterOptions = [
					{ label: LANGUAGE.translate('All Logs'), value: 0 },
					{ label: LANGUAGE.translate('Backup Logs'), value: 1 },
					{ label: LANGUAGE.translate('Download Logs'), value: 2 },
					{ label: LANGUAGE.translate('Restore Logs'), value: 3 },
					{ label: LANGUAGE.translate('System Logs'), value: 5 },
					{ label: LANGUAGE.translate('Notify Logs'), value: 7 },
					{ label: LANGUAGE.translate('Hook Logs'), value: 8 }
				];

				$scope.loaders = {
					stop: false,
					delete: false,
					view:false
				};


				$scope.$watch("filter", function (newVal) {

					$scope.subfilterOptions = [];
					if(!filterSession) $scope.subfilter = "";
					filterSession = 0;
					if($scope.filter !== 1 && $scope.filter !== 6) return;

					switch($scope.filter) {
						case 1:
							$scope.subfilterOptions.push({ label: LANGUAGE.translate("All Backup Jobs"), value: "" });

							jetapi.listBackupJobs({}, function (response) {
								for(var i = 0; i < response.data.jobs.length; i++) {
									$scope.subfilterOptions.push({ label: response.data.jobs[i].name, value: response.data.jobs[i]._id });
								}
							});
						break;
					}

				}, true);


				meta = meta.new("logs");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("start_time");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["pid", "type", "start_time", "end_time", "status"]);
				meta.setTotalItems($scope.logs.length);

				$scope.onClickDownload = function(log) {
					window.location = '/download/?log_id=' + log._id;
				};

				$scope.onClickStop = function(log) {

					if(log.status != 6)
					{
						SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("This log process is not running"));
						return false;
					}

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("The process " + log.pid + " will be killed in the server!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.stop = true;

							jetapi.stopProcess({ _id: log._id }, function(response) {

								$scope.loaders.stop = false;

								if(response.success)
								{
									log.status = 3;
									log.status_name = 'Aborted';
									log.end_time = (new Date()).toISOString();
									log.end_time_int = log.end_time ? parseInt((new Date(log.end_time)).getTime() / 1000) : 0;
									log.execution_time = log.end_time_int && log.start_time_int ? log.end_time_int - log.start_time_int : -1;

									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									return true;
								}

								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Stop log process cancelled"));
						}

					});

				};

				$scope.onClickDelete = function(log) {

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("The selected log will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteLogs({ _id: [log._id] }, function(response) {

								$scope.loaders.delete = false;

								if(response.success)
								{
									$scope.checkall = false;
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									return true;
								}

								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Logs delete cancelled"));
						}

					});
				};

				$scope.onClickView = function(log) {

					$scope.loaders.view = true;

					jetapi.getLog({ _id: log._id, content: 1 }, function (response) {

						$scope.loaders.view = false;

						if(!response.success) {
							SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
							return;
						}

						if(!response.data.content) {
							SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("No log content found"));
							return;
						}

						$uibModal.open({
							animation: true,
							size: "lg",
							ariaLabelledBy: 'modal-title',
							ariaDescribedBy: 'modal-body',
							templateUrl: './app/views/logViewer.ptt',
							controller: 'logViewer',
							controllerAs: '$scope',
							resolve: {
								log: function() { return response.data; }
							}
						});
					});
				};

				$scope.checkChanged = function () {
					for(var i = 0; i < $scope.logs.length; i++)
					{
						if($scope.logs[i].checked && $scope.logs[i].status != 6)
						{
							$scope.checked = true;
							return;
						}
					}

					$scope.checked = false;
				};
				$scope.setCheckAll = function() {
					for(var i = 0; i < $scope.logs.length; i++) {
						var log = $scope.logs[i];
						if(log.status != 6) log.checked = $scope.checkall;
					}
					$scope.checkChanged();
				};
				$scope.deleteSelected = function() {
					var ids = [];

					for(var i in $scope.logs)
					{
						if($scope.logs[i].checked) ids.push($scope.logs[i]._id);
					}

					if(!ids.length)
					{
						SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate("No log IDs selected"));
						return false;
					}

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("The selected logs will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							jetapi.deleteLogs({ _id: ids }, function(response) {

								if(response.success)
								{
									$scope.checkall = false;
									$scope.checked = false;
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									return true;
								}

								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
								return false;
							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Logs delete cancelled"));
						}

					});

				};

				$scope.updateAlert = function(alert) {

					for(var i in $scope.logs)
					{
						if($scope.logs[i]._id === alert._id)
						{
							$scope.logs[i] = alert;
							return false;
						}
					}
				};

				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.loadingLogs = true;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { type: $scope.filter },
						filter: meta.getFilter()
					};

					if($scope.subfilter) {
						switch($scope.filter) {
							case 1: case 6: apiParams.find["info.id"] = $scope.subfilter; break;
						}
					}

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.logs = [];

					jetapi.listLogs(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.logs);
						$scope.logs = response.data.logs;


						for(var i in $scope.logs) {
							$scope.logs[i].start_time_int = $scope.logs[i].start_time ? parseInt((new Date($scope.logs[i].start_time)).getTime() / 1000) : 0;
							$scope.logs[i].end_time_int = $scope.logs[i].end_time ? parseInt((new Date($scope.logs[i].end_time)).getTime() / 1000) : 0;

							if(!$scope.logs[i].end_time_int && $scope.logs[i].status === 6) $scope.logs[i].end_time_int = parseInt((new Date()).getTime() / 1000);
							$scope.logs[i].execution_time = $scope.logs[i].end_time_int && $scope.logs[i].start_time_int ? $scope.logs[i].end_time_int - $scope.logs[i].start_time_int : -1;

							$scope.logs[i].information = [];
							for(var a in $scope.logs[i].info) $scope.logs[i].information.push({ key: a, value: $scope.logs[i].info[a] });
							//if(!data[i].information) data[i].information = '-';
						}

						$scope.loadingLogs = false;
					});
				};

				$timeout($scope.fetchData);
			}
		]
	);
});