'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function(app, LANGUAGE, PERMISSIONS) {
	app.controller("permissions",
		["$rootScope", "$scope", "$routeParams", "$location", "$timeout", "$q", "jetapi", "meta", "SweetAlert", "utils",
			function ($rootScope, $scope, $routeParams, $location, $timeout, $q, jetapi, meta, SweetAlert, utils) {
				$rootScope.$emit('menuItem', 'Permissions');

				$scope.saveing = false;
				$scope.status = undefined;
				$scope.changed = false;
				$scope.loading = false;
				$scope.totalFiltered = 0;
				$scope.selectedAccount = '';

				$scope.details = {};
				$scope.filtered = {};
				$scope.saveData = {};

				$scope.meta = {
					filterType: 1,
					filterValue: ""
				};

				$scope.types = [
					{ value: 1, label: LANGUAGE.translate('Global Permissions') },
					{ value: 2, label: LANGUAGE.translate('Specific Permissions') }
				];

				$scope.fetch = function() {

					if($scope.meta.filterType === 2 && !$scope.selectedAccount) {
						$scope.details = {};
						$scope.filtered = {};
						$scope.saveData = {};
						$scope.totalFiltered = 0;
						return;
					}

					$scope.loading = true;

					var apiParams = {};
					if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

					jetapi.getPermissions(apiParams, function(response) {

						for(var i in response.data.permissions) {
							$scope.details[i] = response.data.permissions[i];
						}

						$scope.totalFiltered = 1;
						$scope.filtered = utils.duplicateObject($scope.details);
						$scope.saveData = utils.duplicateObject($scope.details);
						$scope.loading = false;
					});
				};

				$scope.startFetching = function () {
					$scope.details = {};
					$scope.filtered = {};
					$scope.saveData = {};
					$scope.totalFiltered = 0;
					$scope.fetch();
				};

				$scope.resetPermissions = function () {
					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: ($scope.meta.filterType === 2 && $scope.selectedAccount) ? LANGUAGE.translate("Are you sure you want to reset all permissions for the account \"%s\" to its default settings?", $scope.selectedAccount) : LANGUAGE.translate("Are you sure you want to reset all permissions to the default settings?"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), LANGUAGE.translate("Reset Permissions")]
					}, function(isConfirm){
						if (isConfirm) {

							var apiParams = {};
							if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

							jetapi.resetPermissions(apiParams, function(response) {

								if(response.success) {
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
									$scope.startFetching();
									return;
								}

								SweetAlert.error(LANGUAGE.translate("Error!"), LANGUAGE.translate(response.message));
							});

						}
					});
				};

				$scope.selectAccount = function(account) {
					if(account === undefined) return;
					$scope.selectedAccount = account._id;
					$scope.startFetching();
				};

				$scope.searchAccounts = function(query) {

					if(query)
					{
						var deferred = $q.defer();

						jetapi.listAccounts({ filter: query, sort: { _id: 1 } }, function(response) {

							var results = [];
							for(var i = 0; i < response.data.accounts.length; i++) {
								var account = response.data.accounts[i];
								results.push(account);
							}
							deferred.resolve( results );
						});

						return deferred.promise;
					}

					return [];
				};


				$scope.filteredList = function() {

					$scope.filtered = {};
					$scope.totalFiltered = 0;

					if(!$scope.meta.filterValue) {
						$scope.totalFiltered = 1;
						$scope.filtered = utils.duplicateObject($scope.details);
						return;
					}

					for(var i in $scope.details) {
						var details = PERMISSIONS._get(i);
						if(details === null) continue;

						if(
							($scope.meta.filterValue && !(new RegExp('(' + $scope.meta.filterValue + ')', 'gi')).test(LANGUAGE.translate(details)))
						) continue;

						$scope.totalFiltered++;
						$scope.filtered[i] = $scope.details[i];
					}
				};

				$scope.$on('$destroy', function() {
					if($scope.changed)
					{
						SweetAlert.swal({
							title: LANGUAGE.translate("Are you sure?"),
							text: LANGUAGE.translate("You didn't saved your changes"),
							icon: "warning",
							buttons: [LANGUAGE.translate("Disregard Changes"), LANGUAGE.translate("Save")]
						}, function(isConfirm){
							if (isConfirm) {
								$scope.saveChanges();
							}
						});
					}
				});

				$scope.$watch("searchAccountText", function() {
					if($scope.searchAccountText !== '') return;
					$scope.selectAccount({ _id: '' });
				});

				$scope.$watch("saveData", function(){
					$scope.changed = utils.isChanged($scope.saveData, $scope.details);
				}, true);

				$scope.saveChanges = function(callback) {

					if($scope.saveing) return;

					$scope.status = undefined;
					$scope.saveing = true;

					var apiParams = {
						permissions: utils.saveParams($scope.saveData, $scope.details)
					};

					if($scope.meta.filterType === 2 && $scope.selectedAccount) apiParams.username = $scope.selectedAccount;

					jetapi.managePermissions(apiParams, function(response) {

						$scope.saveing = false;

						if(response.success)
						{
							$scope.changed = false;
							$scope.details = utils.duplicateObject($scope.saveData);

							for(var i in apiParams.permissions) {
								if($scope.filtered[i] !== undefined) $scope.filtered[i] = apiParams.permissions[i];
							}

							if(callback !== undefined) callback();
							SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
						}
						else
						{
							$scope.status = {
								message: LANGUAGE.translate(response.message),
								type: "danger",
								closeable: false,
								ttl: 10000
							};
						}
					});
				};
			}]);
});