'use strict';

define([
	'app',
	'libraries/language'
	], function(app, LANGUAGE) {
	app.controller("queues",
		["$rootScope", "$scope", "$location", "$timeout", "$interval", "SweetAlert", "jetapi", "meta",
			function ($rootScope, $scope, $location, $timeout, $interval, SweetAlert, jetapi, meta) {
				$rootScope.$emit('menuItem', 'Queues');

				$scope.queues = {};
				$scope.loadingQueues = false;
				$scope.showPager = true;
				$scope.cancelling = false;
				$scope.interval = 3;
				$scope.filter = 0;
				$scope.checkInterval = undefined;
				$scope.filterOptions = [
					{ label: LANGUAGE.translate('All Queues'), value: 0 },
					{ label: LANGUAGE.translate('Restore Queues'), value: 1 },
					{ label: LANGUAGE.translate('Download Queues'), value: 2 }
				];

				$scope.cmdType = {
					1: 'restore',
					2: 'downloads'
				};
				meta = meta.new("queues");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("created");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["created","status","type"]);
				meta.setTotalItems($scope.queues.length);

				$scope.startChecking = function() {
					if(angular.isDefined($scope.checkInterval) || parseInt($scope.interval) <= 0) return;
					$scope.checkInterval = $interval(function () {
						$scope.fetchQueues(function(response) {
							var data = response.data.queue;
							for(var i = 0; i < data.length; i++) {
								if($scope.queues[data[i]._id] === undefined) {
									$scope.fetchData();
									return;
								}
								$scope.queues[data[i]._id] = data[i];
							}
						});
					}, parseInt($scope.interval) * 1000);
				};

				$scope.stopChecking = function() {
					if($scope.checkInterval === undefined) return;
					$interval.cancel($scope.checkInterval);
					$scope.checkInterval = undefined;
				};

				$scope.$on("$destroy", function() {
					$scope.stopChecking();
				});

				$scope.$watch("interval", function(){
					$scope.stopChecking();
					$scope.startChecking();
				}, true);

				$scope.rollback = function(queue) {

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("New restore process from the latest safety backup will be inserted to the queue."),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){
						if (isConfirm) {
							jetapi.addQueueRestore({ _id: queue.safety_id }, function (response) {

								if(response.success)
								{
									$scope.queues[response.data._id] = response.data;
									$scope.fetchData();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Queue item process cancelled successfully"));
									return;
								}

								SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
							});
						} else $scope.cancelling = false;
					});

				};

				$scope.cancelQueue = function(queue) {

					if($scope.cancelling) return;

					if(queue.status > 1) {
						SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate("This process already started and can't be cancelled"));
						return;
					}

					$scope.cancelling = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("The selected queue item process will be cancelled"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){
						if (isConfirm) {
							jetapi.cancelQueueItem({ _id: queue._id }, function (response) {

								$scope.cancelling = false;

								if(response.success)
								{
									$scope.queues[response.data._id] = response.data;
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate("Queue item process cancelled successfully"));
									return;
								}

								SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
							});
						} else $scope.cancelling = false;
					});
				};

				$scope.queueIcon = function (queue) {
					if(queue.status === 1) return 'fas fa-clock';
					else if(queue.status > 1 && queue.status < 100) return 'fas fa-cog fa-spin';
					else if(queue.status === 100) return 'fas fa-check';
					else if(queue.status > 100) return 'fas fa-times';
					return '';
				};

				$scope.fetchQueues = function(callback) {

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: { queue_type: $scope.filter },
						filter: meta.getFilter()
					};

					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					jetapi.listQueueItems(apiParams, function(response) {
						if(callback !== undefined) callback(response);
					});
				};
				$scope.$on('fetch', $scope.fetchData);

				$scope.fetchData = function() {

					$scope.stopChecking();
					$scope.loadingQueues = true;
					$scope.queues = {};

					$scope.fetchQueues(function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.queue);
						for(var i = 0; i < response.data.queue.length; i++) $scope.queues[response.data.queue[i]._id] = response.data.queue[i];
						$scope.loadingQueues = false;
						$scope.startChecking();
					});
				};

				$timeout($scope.fetchData);

			}
		]
	);
});