'use strict';

define([
	'app',
	'libraries/language'
	], function(app, LANGUAGE) {
	app.controller("schedules",
		["$rootScope", "$scope", "$location", "$timeout", "jetapi", "meta", "SweetAlert",
			function ($rootScope, $scope, $location, $timeout, jetapi, meta, SweetAlert) {
				$rootScope.$emit('menuItem', 'Schedules');

				$scope.schedules = [];
				$scope.loadingSchedules = false;
				$scope.saveing = false;
				$scope.status = undefined;
				$scope.filter = 0;
				$scope.filterOptions = [
					{ label: LANGUAGE.translate('All Schedules'), value: 0 },
					{ label: LANGUAGE.translate('Manually Schedules'), value: 8 },
					{ label: LANGUAGE.translate('Hourly Schedules'), value: 1 },
					{ label: LANGUAGE.translate('Daily Schedules'), value: 2 },
					{ label: LANGUAGE.translate('Weekly Schedules'), value: 3 },
					{ label: LANGUAGE.translate('Monthly Schedules'), value: 4 },
					{ label: LANGUAGE.translate('After Backup Job Schedules'), value: 5 }
				];

				$scope.loaders = {
					delete: false
				};

				meta = meta.new("schedules");
				$scope.meta = meta;
				$scope.metaData = meta.getData();

				if(!meta.getSortBy()) meta.setSortBy("name");
				if(!meta.getSortDirection()) meta.setSortDirection("desc");
				meta.setSortFields(["name","type","owner","jobs_count"]);
				meta.setTotalItems($scope.schedules.length);

				$scope.onClickDelete = function(schedule) {

					if($scope.saveing) return;

					if(schedule.jobs_count > 0)
					{
						SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate("This schedule has an assigned jobs, Please remove those jobs before deleting this schedule."));
						return false;
					}

					$scope.saveing = true;

					SweetAlert.swal({
						title: LANGUAGE.translate("Are you sure?"),
						text: LANGUAGE.translate("This schedule will be permanently deleted!"),
						icon: "warning",
						buttons: [LANGUAGE.translate("Cancel"), true]
					}, function(isConfirm){

						if (isConfirm)
						{
							$scope.loaders.delete = true;

							jetapi.deleteSchedule({ _id: schedule._id }, function(response) {

								$scope.saveing = false;
								$scope.loaders.delete = false;

								if(response.success)
								{
									fetch();
									SweetAlert.success(LANGUAGE.translate("Success!"), LANGUAGE.translate(response.message));
								}
								else
								{
									SweetAlert.error(LANGUAGE.translate("Failed!"), LANGUAGE.translate(response.message));
								}

							});
						}
						else
						{
							$scope.saveing = false;
							SweetAlert.error(LANGUAGE.translate("Cancelled!"), LANGUAGE.translate("Schedule delete cancelled"));
						}

					});
				};

				$scope.$on('fetch', fetch);

				function fetch() {

					$scope.loadingSchedules = true;
					$scope.status = undefined;

					var apiParams = {
						sort: {},
						skip: meta.getSkip(),
						limit: meta.getPageSize(),
						find: {},
						filter: meta.getFilter()
					};

					if($scope.filter) apiParams.find.type = $scope.filter;
					apiParams.sort[meta.getSortBy()] = meta.getSortDirectionInt();

					$scope.schedules = [];

					jetapi.listSchedules(apiParams, function(response) {
						meta.setTotalItems(response.data.total);
						meta.calculate(response.data.schedules);
						$scope.schedules = response.data.schedules;
						$scope.loadingSchedules = false;
					});
				}

				$timeout(fetch);
			}
		]
	);
});