'use strict';

define([
	'app',
	'libraries/language',
	'libraries/permissions'
	], function (app, LANGUAGE, PERMISSIONS) {
	app.directive('navigation', ["$location", "$rootScope", function($location, $rootScope) {

		return {

			template: '' +
				'<li aria-label="{{ option.label }}" ng-repeat="option in options" ng-if="option.permissions" ng-class="{ \'active\': active == option.class }" >'+
					'<a  href="{{url}}{{option.href}}" ng-click="changePath(option.href)" tooltip-placement="right" tooltip-enable="status == \'collapse\'" uib-tooltip="{{ option.label }}" >'+
						'<em aria-hidden="true" class="{{ option.icon }}"></em>'+
						'<span class="aatest">{{ option.label }}</span>'+
					'</a>'+
				'</li>',
			scope: {
				"status": "=",
				"active": "=",
				"logout": "="
			},
			link: function (scope) {

				scope.changePath = function(path) {
					$location.path(path)
				};

				scope.url = $rootScope.primaryURL;
				scope.options = [
					{label: LANGUAGE.translate("Dashboard"), href: "/", class: 'Dashboard', icon: 'fas fa-home', permissions: true },
					{label: LANGUAGE.translate("Users"), href: "/accounts", class: 'Accounts', icon: 'fas fa-users', permissions: PERMISSIONS.isRoot },
					{label: LANGUAGE.translate("Destinations"), href: "/destinations", class: 'Destinations', icon: 'fas fa-folder', permissions: PERMISSIONS.canManageDestinations },
					{label: LANGUAGE.translate("Database Connections"), href: "/databases", class: 'Databases', icon: 'fas fa-database', permissions: PERMISSIONS.canManageBackupJobs },
					{label: LANGUAGE.translate("Backup Jobs"), href: "/backupJobs", class: 'BackupJobs', icon: 'fas fa-cubes', permissions: PERMISSIONS.canManageBackupJobs },
					{label: LANGUAGE.translate("Restore & Download"), href: "/backupManager", class: 'BackupManager', icon: 'fas fa-sync', permissions: PERMISSIONS.canManageBackups },
					{label: LANGUAGE.translate("Alerts"), href: "/alerts", class: 'Alerts', icon: 'fas fa-exclamation-triangle', permissions: PERMISSIONS.isRoot },
					{label: LANGUAGE.translate("Queues"), href: "/queues", class: 'Queues', icon: 'fas fa-clock', permissions: PERMISSIONS.canManageQueue },
					{label: LANGUAGE.translate("Logs"), href: "/logs", class: 'Logs', icon: 'fas fa-file-alt', permissions: PERMISSIONS.isRoot },
					{label: LANGUAGE.translate("Permissions"), href: "/permissions", class: 'Permissions', icon: 'fas fa-lock', permissions: PERMISSIONS.isRoot },
					{label: LANGUAGE.translate("Hooks"), href: "/hooks", class: 'Hooks', icon: 'fas fa-flag', permissions: PERMISSIONS.isRoot },
					//{label: LANGUAGE.translate("Plugins"), href: "/plugins", class: 'Plugins', icon: 'fas fa-puzzle-piece', permissions: PERMISSIONS.isRoot },
					{label: LANGUAGE.translate("Settings"), href: "/settings", class: 'Settings', icon: 'fas fa-cog', permissions: PERMISSIONS.isRoot }
				];
			}
		};
	}]);
});

