'use strict';

define([
	'libraries/consts'
], function(CONSTS) {
	var UTILS = function() {};

	UTILS.prototype = {
		sizeToHumanReadable: function(bytes, si, decimals) {
			return (this.sizeToNumber(bytes, si, decimals) + " " + this.sizeToType(bytes, si)).trim();
		},
		sizeToHumanHz: function(bytes, decimals) {
			return (this.sizeToNumber(bytes, true, decimals) + " " + this.sizeToHz(bytes)).trim();
		},
		sizeToNumber: function(bytes, si, decimals) {
			if(si === undefined) si = false;
			if(decimals === undefined) decimals = 2;
			var unit = si ? 1000 : 1024;
			if (bytes < unit) return bytes;
			var exp = parseInt(Math.log(bytes)/Math.log(unit));
			return (bytes/Math.pow(unit, exp)).toFixed(decimals);
		},
		timeParsed: function(time, includeSeconds) {
			if(includeSeconds === undefined) includeSeconds = false;
			var hours = Math.floor(time / 3600);
			if(hours < 10) hours = "0" + hours;
			var minutes = Math.floor((time / 60) % 60);
			if(minutes < 10) minutes = "0" + minutes;
			var seconds = time % 60;
			if(seconds < 10) seconds = "0" + seconds;
			return hours + ":" + minutes + (includeSeconds ? ":" + seconds : '');
		},
		sizeToType: function(bytes, si) {
			if(si === undefined) si = false;
			var unit = si ? 1000 : 1024;
			if (bytes < unit) return "B";
			var exp = parseInt(Math.log(bytes)/Math.log(unit));
			var pre = (si ? "kMGTPE" : "KMGTPE");
			return pre[exp-1] + "B";
		},
		sizeToHz: function(bytes) {
			var unit = 1000;
			if (bytes < unit) return "";
			var exp = parseInt(Math.log(bytes)/Math.log(unit));
			var pre = "MGTPE";
			return pre[exp-1] + "Hz";
		},
		versionCompare: function(v1, v2) {
			var v1parts = v1.split('.'),
				v2parts = v2.split('.');

			function isValidPart(x) { return /^\d+$/.test(x); }

			if (!v1parts.every(isValidPart) || !v2parts.every(isValidPart)) return NaN;

			v1parts = v1parts.map(Number);
			v2parts = v2parts.map(Number);

			for (var i = 0; i < v1parts.length; ++i) {
				if (v2parts.length == i) return 1;
				if (v1parts[i] == v2parts[i]) continue;
				else if (v1parts[i] > v2parts[i]) return 1;
				else return -1;
			}

			if (v1parts.length != v2parts.length) return -1;

			return 0;
		},
		checkMinimumJBInstallation: function(version) {
			return this.versionCompare(version, CONSTS.MINIMUM_JB_INSTALLATION);
		},
		getRandomColor: function() {
			var letters = '0123456789ABCDEF';
			var color = '#';
			for (var i = 0; i < 6; i++) {
				color += letters[Math.floor(Math.random() * 16)];
			}
			return color.toLowerCase();
		},
		increaseColorBrightness: function(hex, percent){

			hex = hex.replace(/^\s*#|\s*$/g, '');

			if(hex.length == 3){
				hex = hex.replace(/(.)/g, '$1$1');
			}

			var r = parseInt(hex.substr(0, 2), 16),
				g = parseInt(hex.substr(2, 2), 16),
				b = parseInt(hex.substr(4, 2), 16);

			return '#' +
				((0|(1<<8) + r + (256 - r) * percent / 100).toString(16)).substr(1) +
				((0|(1<<8) + g + (256 - g) * percent / 100).toString(16)).substr(1) +
				((0|(1<<8) + b + (256 - b) * percent / 100).toString(16)).substr(1);
		},
		countObj: function (obj) {
			var count = 0;
			for(var key in obj) count++;
			return count;
		}
	};

	return new UTILS();
});